<?php
/***************************************************************************************************************************************************

	Purpose: to handle all logic involved in writing a review
	
	History:
		File added on 07/19/2007
		File modified on 06/23/2008 - for reCAPTCHA support
		File modified on 11/26/2010 - for XSS vulnerabilitiy in $review[Rating]

***************************************************************************************************************************************************/
// make sure this file isn't accessed directly via a browser
if (!defined('SC_INCLUDE_OK')) die;

/*******************************************************************************************
	Setup vars common to both review form, review thanks, and review banned templates
*******************************************************************************************/
$Write_Review_Header_Image 	= sc_img('review_write_header');


/*************************************************************************
	Make sure they are logged in
*************************************************************************/
if (!$SC['user']) {

	// setup xtra account choice fields, that will cause review_write.php to be included
	$xtra_acct_choice_fields = array(
		'p'			=> is_numeric($_REQUEST['p']) ? $_REQUEST['p'] : '', 
	);
	if (!empty($_REQUEST['review'])) {
		// if here, review operation is initiated
		$xtra_acct_choice_fields["review[action]"] 	= sc_htmlspecialchars($_REQUEST['review']['action']);
		$xtra_acct_choice_fields["review[prod_rn]"] = is_numeric($_REQUEST['review']['prod_rn']) ? $_REQUEST['review']['prod_rn'] : 0;
	}

	
	
	if ($_GET['acct_choice']['do'] == 'new' || $_POST['acct']) {
		
		/*************************************************			
			Show account form
		*************************************************/			
		$acct_result 				= sc_account($xtra_acct_choice_fields);
		$Account_HTML 				= $acct_result['html'];
		$Error_Message 				= !empty($acct_result['error_msgs']) 	? implode('<br/>',$acct_result['error_msgs']) 	: '';
		$Info_Message 				= !empty($acct_result['info_msgs']) 	? implode('<br/>',$acct_result['info_msgs']) 	: '';
		$Account_Message			= !empty($acct_result['success_msgs'])	? $acct_result['success_msgs'][0]				: '';

		// if user successfully created an account
		if ($acct_result['status'] == 'ok') {
			// POST data ($_POST['review']) will cause conflict with code further down,
			// making it think review form is being posted...this prevents that, but keeps it in
			// $_REQUEST where it is still needed
			unset($_POST['review']);
			
		} else {
			// show form to create one
			// add it into a template 
			ob_start();
			include sc_tpl('review_write_login');
			$SC_content['review'] = ob_get_clean();

			return;
		}	
	
	} elseif (!empty($_POST['acct_choice'])) {
		/*************************************************			
			Process login info from account choice form
		*************************************************/			
		// make sure all required fields have been passed
		if (!$_POST['acct_choice']['login']['user']) $error_msgs[] = 'The "username" field is required';
		if (!$_POST['acct_choice']['login']['pass']) $error_msgs[] = 'The "password" field is required';
		
		// validate token to prevent CSRF attacks
		if ($_POST['token'] != $SC['token']) $error_msgs[] = 'Security token mismatch. Please contact merchant for assistance.'; 	
		
		// if all info passed, try to login
		if (empty($error_msgs)) {
			$login_ok = sc_login_process(
				array(
					'username'		=> $_POST['acct_choice']['login']['user'],
					'password'		=> $_POST['acct_choice']['login']['pass'],
					'remember_me'	=> $_POST['acct_choice']['login']['remember'],
				),
				1
			);
			
			// see if login was OK or not
			if (!$login_ok) {
				$error_msgs[] = 'Login attempt failed';
			} elseif (!empty($SC['user'])) {
				$Account_Message = 'Login successful';

				// wipe out generic msg
				unset($SC['msg_generic']);
				
				// POST data ($_POST['review']) will cause conflict with code further down,
				// making it think review form is being posted...this prevents that, but keeps it in
				// $_REQUEST where it is still needed
				unset($_POST['review']);
			}
		}
		
		// if login failed, let them know
		if (!empty($error_msgs)) {
			$Account_HTML 			= sc_account_choice(1, $xtra_acct_choice_fields);
			$Info_Message 			= str_replace("SC_IMG_DIR_DYN",SC_IMG_DIR_DYN,$SC['modules']['review']['settings']['Login_Required_Message']);
			$Error_Message			= implode('<br/>',$error_msgs);

			// add it into a template 
			ob_start();
			include sc_tpl('review_write_login');
			$SC_content['review'] = ob_get_clean();
			
			return;	
		}
		
	} else {	
	
		/*********************************
			Grab account choice HTML
		*********************************/
		$Account_HTML 	= sc_account_choice(1, $xtra_acct_choice_fields);
		$Info_Message 	= $SC['modules']['review']['settings']['Login_Required_Message'];
		
		
		// add it into a template 
		ob_start();
		include sc_tpl('review_write_login');
		$SC_content['review'] = ob_get_clean();
		
		return;	
	}
}

// user account may have been established above...check again
if (!isset($SC['user'])) return;

if (get_field_val('Reviewer_Settings','Banned',"record_number = '{$SC['user']['record_number']}'")) {
	
	// If here, user is banned from writing reviews 
	
	// grab banned template and bail
	ob_start();
	include sc_tpl('review_write_banned');
	$SC_content['review'] = ob_get_clean();


	return;	
}





/*************************************************************************
	User is logged in...handle writing review
*************************************************************************/

if (!empty($_POST['review'])) {

	$review = $_POST['review'];

} elseif (!empty($_GET['review'])) {
	// if here, review operation is initiated
	$review['action'] 	= $_GET['review']['action'];
	$review['edit'] 	= $_GET['review']['edit'];
	$review['prod_rn']	= $_GET['review']['prod_rn'];
}

$reviewer_settings	= $_REQUEST['reviewer_settings'] ? $_REQUEST['reviewer_settings'] : $SC['user']['reviewer_settings'];

// remove slashes if magic quotes is on :(
if (get_magic_quotes_gpc()) {
	if (is_array($review)) 				foreach($review 			as $key => $value) $review[$key] 				= stripslashes($value);
	if (is_array($reviewer_settings)) 	foreach($reviewer_settings 	as $key => $value) $reviewer_settings[$key] 	= stripslashes($value);
}


// put prod rn in $review if its being passed
if (is_numeric($_REQUEST['p'])) $review['prod_rn'] = $_REQUEST['p'];

// if we don't have a product record, get the hell out of dodge
if (!is_numeric($review['prod_rn'])) {
	$SC['msg_generic'] = sc_msg('Error: unable to continue. Product record number unavailable.', 'error');
	return;
}



// Vars for use in templates
$Moderation_On				= $SC['modules']['review']['settings']['Moderate'] && !$SC['user']['reviewer_settings']['Moderation_Exempt'];
$Product_Name 				= htmlspecialchars(sc_query("SELECT `Name` FROM `Products` WHERE `record_number` = '$review[prod_rn]'"));




/************************************************************************
	If user is attempting an edit, put that review's info in $review
************************************************************************/
if (is_numeric($_GET['review']['edit'])) {

	// if edits not allowed
	if (!$SC['modules']['review']['settings']['Edits_Allowed']) {
		$Error_Message = 'Review editing is disabled.';
	} else {
	
		// grab review IF it belongs to this customer
		$review_record = sc_query("
			SELECT
				 `Products_rn` AS `prod_rn`
				,`Rating`
				,`Title`
				,`Review`
				,`Date`
			FROM
				`Reviews`
			WHERE
				`record_number` = '$review[edit]'
				AND `Customer_rn` = '{$SC['user']['record_number']}'
		");
		
		if (is_array($review_record)) {
			// Review found... check to see if timeframe is within allowed timespan
			$review_record['seconds_elapsed'] = time() - strtotime($review_record['Date']);

			if ($SC['modules']['review']['settings']['Edits_Seconds'] < $review_record['seconds_elapsed']) {
				// if we've past the cutoff date, don't allow edit
				$Error_Message = 'The date on this review is past the period in which edits are allowed.';	
			} else {
				
				// save review record in cache so we can read it on save
				$SC['review_record'] = $review_record;
				
				// put info in $review so it shows up on form
				$review = array_merge($review,$review_record);
			}
		} else {
			// review not found
			$Error_Message = 'Error: Review not found.';
		}
	}
	
	// show error if any
	if ($Error_Message) {
		ob_start();
		include sc_tpl('review_write_error');
		$SC_content['review'] = ob_get_clean();

		
		return;
	}
}


$Current_Rating_Saved		= $review['Rating'] ? 'background-position: left bottom' : '';	// this makes the default stars appear selected



// grab info about all reviews
$reviews_info = sc_query("
	SELECT 
		 COUNT(`record_number`) AS `Total_Votes`
		,SUM(`Rating`) / COUNT(`record_number`) AS `Average_Rating`
	FROM 
		`Reviews`
	WHERE 
		`Products_rn` = '$review[prod_rn]' 
		AND `Status` = 'approved'
	"
);

// format avg. rating to 2 decimal places
$reviews_info['Average_Rating'] = round($reviews_info['Average_Rating'],2);

// add avg. rating percent to control width of star rater
$reviews_info['Average_Rating_Percentage'] = round(($reviews_info['Average_Rating'] * 20));

extract($reviews_info);

$Total_Votes_s = $Total_Votes == 1 ? '' : 's'; 

$Product_HREF 	= get_product_url($review['prod_rn']);

// current rating will either be set to the overall average (when customer hasn't rated item), OR their rating when they have rated the item
if (is_numeric($review['Rating']) && $review['Rating'] > 0 && $review['Rating'] < 6) {
	// if here, customer submitted a rating
	$Current_Rating_Percentage 	= ($review['Rating'] * 20).'%';
	$Current_Rating_Number 		= $review['Rating'];
	$Your_Rating				= $review['Rating'];
	$Your_Rating_Display		= 'display: block';
} else {
	$Current_Rating_Percentage 	= $Average_Rating_Percentage.'%';
	$Current_Rating_Number 		= $Average_Rating;

	// IE 6 has yet another bug that doesn't permit us to hide and then display the "your rating:" text...
	// so, only hide it by default for all but IE 6
	$Your_Rating_Display		= ($SC['browser']['browser'] == 'ie' && $SC['browser']['maj_ver'] <= 6) ? '' : 'display: none';
	
	unset($review['Rating']);
}			



/****************************************************************************************
	If user already submitted a review, don't let that sneaky monkey write another one
****************************************************************************************/
// grab rn of prior review, if it exists
$existing_review_rn = sc_query("SELECT `record_number` FROM `Reviews` WHERE `Products_rn` = '$review[prod_rn]' AND `Customer_rn` = '{$SC['user']['record_number']}'");

// if existing review found, and it is not being edited
if (is_numeric($existing_review_rn) && $existing_review_rn != $review['edit']) {
	
	$Error_Message = 'You have already submitted a review for this item.';
	
	ob_start();
	include sc_tpl('review_write_error');
	$SC_content['review'] = ob_get_clean();

	
	return;
}



/*************************************************************
	Handle posted review form
*************************************************************/
if (is_array($_POST['review'])) {
	$errors 		= array();
	$error_fields 	= array();

	// validate rating
	if (!is_numeric($review['Rating']) || $review['Rating'] < 1 || $review['Rating'] > 5) {
		$errors[] 			= "'Rating' is required.";
		$error_fields[] 	= 'Rating';
		unset($review['Rating']);
	}
	
	// validate Title
	if (!$review['Title']) {
		$errors[] 			= "'Title' is required.";
		$error_fields[] 	= 'Title';
	} elseif (strlen($review['Title']) < $SC['modules']['review']['settings']['Min_Title_Length'] || strlen($review['Title']) > $SC['modules']['review']['settings']['Max_Title_Length']) {
		// if here, title isn't within min/max setting
		$errors[]			= "'Title' must be between {$SC['modules']['review']['settings']['Min_Title_Length']} and {$SC['modules']['review']['settings']['Max_Title_Length']} characters in length.";
		$error_fields[] 	= 'Title';
	} elseif (!valid_non_html($review['Title'])) {
		$errors[]			= "'Title' cannot contain HTML.";
		$error_fields[] 	= 'Title';
	}

	// validate Review
	if (!$review['Review']) {
		$errors[] 			= "'Review' is required.";
		$error_fields[] 	= 'Review';
	} elseif (strlen($review['Review']) < $SC['modules']['review']['settings']['Min_Review_Length'] || strlen($review['Review']) > $SC['modules']['review']['settings']['Max_Review_Length']) {
		// if here, title isn't within min/max setting
		$errors[]			= "'Review' must be between {$SC['modules']['review']['settings']['Min_Review_Length']} and {$SC['modules']['review']['settings']['Max_Review_Length']} characters in length.";
		$error_fields[] 	= 'Review';
	} elseif (!valid_non_html($review['Review'])) {
		$errors[]			= "'Review' cannot contain HTML.";
		$error_fields[] 	= 'Review';
	}
	
	// validate reviewer settings, if on form
	if (strlen($reviewer_settings['Name'])) {
		if (strlen($reviewer_settings['Name']) < $SC['modules']['review']['settings']['Min_Reviewer_Name_Length'] || strlen($reviewer_settings['Name']) > $SC['modules']['review']['settings']['Max_Reviewer_Name_Length']) {
			// if here, Name isn't within min/max setting
			$errors[]			= "'Name' must be between {$SC['modules']['review']['settings']['Min_Reviewer_Name_Length']} and {$SC['modules']['review']['settings']['Max_Reviewer_Name_Length']} characters in length.";
			$error_fields[] 	= 'RS_Name';
		} elseif (!valid_non_html($reviewer_settings['Name'])) {
			$errors[]			= "'Name' cannot contain HTML.";
			$error_fields[] 	= 'RS_Name';
		}
	}
	if (strlen($reviewer_settings['Location'])) {
		if (strlen($reviewer_settings['Location']) < $SC['modules']['review']['settings']['Min_Reviewer_Location_Length'] || strlen($reviewer_settings['Location']) > $SC['modules']['review']['settings']['Max_Reviewer_Location_Length']) {
			// if here, Location isn't within min/max setting
			$errors[]			= "'Location' must be between {$SC['modules']['review']['settings']['Min_Reviewer_Location_Length']} and {$SC['modules']['review']['settings']['Max_Reviewer_Location_Length']} characters in length.";
			$error_fields[] 	= 'RS_Location';
		} elseif (!valid_non_html($reviewer_settings['Location'])) {
			$errors[]			= "'Location' cannot contain HTML.";
			$error_fields[] 	= 'RS_Location';
		}
	}
	
	// check recaptcha
	if ($SC['settings']['reCAPTCHA_Enabled'] && $SC['settings']['reCAPTCHA_Review_Form']) {
		$recaptcha_error = '';
		$privatekey = $SC['settings']['reCAPTCHA_Private_Key'];
        $resp = recaptcha_check_answer ($privatekey,
                                        $_SERVER['REMOTE_ADDR'],
                                        $_POST['recaptcha_challenge_field'],
                                        $_POST['recaptcha_response_field']);
        if (!$resp->is_valid) {
			// set the error code so that we can display it
			$recaptcha_error = $resp->error;

			// If captcha not filled out
			$errors[] = "<div class=\"error\">The words entered in the Spam Filter did not match.</div>";
        }
	}
	
	
	// if we have any errors (duh!)
	if (count($errors)) {
		$Error_Message = 'Please correct the following errors: <br/>'.implode('<br/>',$errors);

		// setup error icons and classes
		foreach($error_fields as $error_field) {
			${$error_field.'_Class'} 	= 'error_field';	// used to highlight Title field
			${$error_field.'_Icon'} 	= $SC['settings']['Missing_Field_Indicator'];
		}
	} else {
		/***********************************************************************************************************
			No errors....add review to database and thank customer
		***********************************************************************************************************/
		// setup data array for review record
		$review_record = array(
			'Products_rn'	=> $review['prod_rn'],
			'Customer_rn'	=> $SC['user']['record_number'],
			'Title'			=> $review['Title'],
			'Rating'		=> $review['Rating'],
			'Review'		=> $review['Review'],
			'Date'			=> sc_mysql_date(),
			'Status'		=> $Moderation_On ? 'pending' : 'approved',
		);

		if (is_numeric($review['edit'])) {
			// do another check here to make sure user is allowed to edit this record, just in case
			if (!$SC['modules']['review']['settings']['Edits_Allowed'])	{
				$Error_Message = 'Review editing is disabled.';
			} elseif ($SC['modules']['review']['settings']['Edits_Seconds'] < $SC['review_record']['seconds_elapsed']) {
				// if we've past the cutoff date, don't allow edit
				$Error_Message = 'The date on this review is past the period in which edits are allowed.';	
			}
			modify_record('Reviews',$review_record,"`record_number` = '$review[edit]'");
		} else {
			$review_rn = add_record('Reviews',$review_record);
		}
		
		// $review_rn is used in moderation email...if its not set but review was edited, use edited review's rn instead
		if (!is_numeric($review_rn)) $review_rn = $review['edit'];
		
		// add user's rating in with others
		$Average_Rating				= round((($Average_Rating * $Total_Votes) + $review['Rating']) / ($Total_Votes + 1),2);
		$Total_Votes				= $Total_Votes + 1;
		$Total_Votes_s				= $Total_Votes > 1 ? 's' : '';

		// this shouldn't really be needed, but just in case remove any special chars
		foreach($review as $key => $value) $review[$key] = htmlspecialchars(stripslashes($value));
		
		// add line breaks for display
		$Review_Text 		= $review['Review']; // save review in plain text for moderator text email
		$review['Review'] 	= nl2br($review['Review']);
		
		// add or update reviewer settings
		if ($reviewer_settings) {
			// add settings
			$rv_settings_record = array(
				'record_number'		=> $SC['user']['record_number'],
				'Name'				=> $reviewer_settings['Name'],
				'Location'			=> $reviewer_settings['Location'],
				'Email_Moderation'	=> $reviewer_settings['Email_Moderation'],
				'Approved'			=> $Moderation_On ? 0 : 1,
			);
			
			$settings_exist = get_field_val('Reviewer_Settings','record_number',"record_number = '{$SC['user']['record_number']}'");
			
			if ($settings_exist) {
				// update settings
				modify_record('Reviewer_Settings',$rv_settings_record,"`record_number` = '{$SC['user']['record_number']}'");
			} else {
				$review_record['record_number'] = add_record('Reviewer_Settings',$rv_settings_record);
			}
		}

		ob_start();
		include sc_tpl('review_write_thanks');
		$SC_content['review'] = ob_get_clean();

		
		
		/*****************************************************************************************************
			Notify moderator if moderation is enabled AND if we have email addresses to send notice to
		*****************************************************************************************************/
		if ($Moderation_On && $SC['modules']['review']['settings']['Moderator_Email']) {
			
			// grab email addresses in array, arranged by email preference
			$moderator_addresses = sc_email_addresses($SC['modules']['review']['settings']['Moderator_Email']);
			
			// only send message if we have addresses
			if (count($moderator_addresses)) {
				// turn address array into a CSV string, one for each email type
				$text_email_to = count($moderator_addresses['text']) ? implode(',',$moderator_addresses['text']) : false;
				$html_email_to = count($moderator_addresses['html']) ? implode(',',$moderator_addresses['html']) : false;

				// setup vars with path to template files
				$text_tpl 			= sc_tpl('review_moderator_email_txt');
				$html_tpl 			= sc_tpl('review_moderator_email_htm');
				
				// Setup vars for use in template files
				$Product_HREF 					= get_product_url($review['prod_rn'],0,1);					// $Product_HREF may contain the customer's SID!...this sets the var again without it
				$Approve_Review_URL 			= SC_CP_PAGE_DYN."?review=approve&rn=$review_rn";		// URLs used to approve/reject review in control panel
				$Reject_Review_URL 				= SC_CP_PAGE_DYN."?review=reject&rn=$review_rn";
				$Edit_Review_URL 				= SC_CP_PAGE_DYN."?edit_records=1&selected_record_number=$review_rn&table=Reviews";
				$Review_CSS_URL					= is_file(SC_THEME_DIR.'/review.css.php') ? $SC['www_theme_dir'].'/review.css.php' : $SC['modules']['review']['www_root'].'/templates/review.css.php';
				$Pending_Review_Header_Image 	= sc_img('review_pending_header','dyn');
				
				// extract vars for use in template
				extract($review);
				extract($reviewer_settings);
				
				// Set review name and location if they are using the default from their user account
				if (!$Name) 	$Name = $SC['user']['First_Name'].' '.$SC['user']['Last_Name'];
				if (!$Location) {
					// get state, OR state other
					$state 		= $SC['user']['State_or_Province'] ? get_field_val('States','Name',"record_number = '{$SC['user']['State_or_Province']}'") : $SC['user']['State_Other'];
					$country	= get_field_val('Countries','Name',"record_number = '{$SC['user']['Country']}'");
					$Location	= "$state - $country";
				}
				
				// setup text email...store in var
				ob_start();
				include $text_tpl;
				$text_email = ob_get_clean();


				// handle HTML email...only need to do this if at least one moderator has HTML set as their email type preference
				$html_email = '';
				if (strlen($html_email_to)) {
					// grab HTML email and store in var
					ob_start();
					include $html_tpl;
					$html_email = ob_get_clean();

					
					// send...include text email in case they have HTML disabled
					sc_email($html_email_to,'Review Pending Moderation',$text_email,"Reviewer: $Name",$SC['user']['Email_Address'],0,0,$html_email,1);
				}

				if (strlen($html_email_to)) {
					// send text email, only if at least one moderator prefers text
					sc_email($text_email_to,'Review Pending Moderation',$text_email,"Reviewer: $Name",$SC['user']['Email_Address'],0,0,0,1);
				}
			}
		}
		
		return;
	}
	
}








/*************************************************************
	Show form to write review
*************************************************************/
// Setup up vars for us in review_write template
$Form_Action 			= SC_STORE_PAGE_HTTP;
$Title_Max_Length		= $SC['modules']['review']['settings']['Max_Title_Length'];
$Name_Max_Length		= $SC['modules']['review']['settings']['Max_Reviewer_Name_Length'];
$Location_Max_Length	= $SC['modules']['review']['settings']['Max_Reviewer_Location_Length'];

// Grab reCAPTCHA field
$reCAPTCHA_HTML = $SC['settings']['reCAPTCHA_Enabled'] && $SC['settings']['reCAPTCHA_Review_Form'] ? sc_recaptcha($recaptcha_error) : '';


// set the selected rating for the non JS rater (radio buttons)
unset($Selected_Rating);
if (is_numeric($review[Rating])) $Selected_Rating[$review[Rating]] = 'checked';

// current rating
$Star_Rater_ID 				 = microtime();	// ID to be used for the UL element of the star rater
$Hidden_Fields				 = SID_FLD_SSL."<input type=\"hidden\" name=\"review[action]\"	value=\"write\" />
								<input type=\"hidden\" name=\"review[prod_rn]\" value=\"$review[prod_rn]\" />
								<input type=\"hidden\" name=\"review[Rating]\"	value=\"$review[Rating]\" id=\"{$Star_Rater_ID}_rating\" />";
if (is_numeric($review['edit'])) $Hidden_Fields .= "<input type=\"hidden\" name=\"review[edit]\" value=\"$review[edit]\" />";


// prep data that was posted to go in form
if (is_array($review)) 				foreach($review as $key => $value) 				$review[$key] 				= htmlspecialchars(stripslashes($value));
if (is_array($reviewer_settings)) 	foreach($reviewer_settings as $key => $value) 	$reviewer_settings[$key] 	= htmlspecialchars(stripslashes($value));

// state for Email_Moderation field
$Email_Moderation_Checked = $reviewer_settings['Email_Moderation'] ? 'checked' : '';

ob_start();
include sc_tpl('review_write');
$SC_content['review'] = ob_get_clean();

?>